using System;
using System.Collections.Generic;
using System.Text;
using Silvermoon.GdiPlus.Native;
using Color = System.Drawing.Color;

namespace Silvermoon.GdiPlus
{
    public class PenPlus: IDisposable
    {
        public PenPlus(Color color, float width)
        {
            Unit unit = Unit. UnitWorld;
            nativePen = null;
            lastResult =  Silvermoon.GdiPlus.Native.GdiPlus.GdipCreatePen1(color.ToArgb(), width, unit, out nativePen);
        }

        public PenPlus(BrushPlus brush, float width)
        {
            Unit unit = Unit.UnitWorld;
            nativePen = null;
            lastResult = Silvermoon.GdiPlus.Native.GdiPlus.GdipCreatePen2(brush.nativeBrush, width, unit, out nativePen);
        }
        public PenPlus(Color color, float width, bool opaque)
        {
            int c = color.ToArgb();
            if (opaque) c |= (0xff << 24);
            Unit unit = Unit.UnitWorld;
            nativePen = null;
            lastResult =  Silvermoon.GdiPlus.Native.GdiPlus.GdipCreatePen1(c, width, unit, out nativePen);
        }

        ~PenPlus()
        {
            Dispose(true);
        }

        public PenPlus Clone()
        {
            GpPen clonePen = null;

            lastResult =  Silvermoon.GdiPlus.Native.GdiPlus.GdipClonePen(nativePen, out clonePen);

            return new PenPlus(clonePen, lastResult);
        }

        public float Width
        {
            get { return GetWidth(); }
     //       set { SetWidth(value); }
        }

        public Color Color
        {
            get { return GetColor(); }
            set { SetColor(value); }
        }

        public CustomLineCap CustomEndCap
        {
            get { return GetCustomEndCap(); }
            set { SetCustomEndCap(value); }
        }

        public CustomLineCap CustomStartCap
        {
            get { return GetCustomStartCap(); }
            set { SetCustomStartCap(value); }
        }

        public DashCap DashCap
        {
            get { return GetDashCap(); }
        }

        public LineCap EndCap
        {
            get { return GetEndCap(); }
            set { SetEndCap(value); }
        }

        public LineCap StartCap
        {
            get { return GetStartCap(); }
            set { SetStartCap(value); }
        }

        public float DashOffset
        {
            get { return GetDashOffset(); }
            set { SetDashOffset(value); }
        }

        public int DashPatternCount
        {
            get { return GetDashPatternCount(); }
        }

        public PenAlignment Alignment
        {
            get { return GetAlignment(); }
            set { SetAlignment(value); }
        }
        

        public LineJoin LineJoin
        {
            get { return GetLineJoin(); }
            set { SetLineJoin(value); }
        }

        public float MiterLimit
        {
            get { return GetMiterLimit(); }
            set { SetMiterLimit(value); }
        }

        private void SetStatus(GpStatus gpStatus)
        {
            if (gpStatus != GpStatus.Ok) throw GdiPlusStatusException.Exception(gpStatus);
        }

        float GetWidth()
        {
            float width;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenWidth(nativePen, out width));

            return width;
        }



        // Set/get line caps: start, end, and dash

        // Line cap and join APIs by using LineCap and LineJoin enums.

        public void SetLineCap(LineCap startCap,
                          LineCap endCap,
                          DashCap dashCap)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenLineCap197819(nativePen,
                                       startCap, endCap, dashCap));
        }

        void SetStartCap(LineCap startCap)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenStartCap(nativePen, startCap));
        }

        void SetEndCap(LineCap endCap)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenEndCap(nativePen, endCap));
        }

        //public void SetDashCap(DashCap dashCap)
        //{
        //    SetStatus(NativeMethods.GdipSetPenDashCap197819(nativePen,
        //                               dashCap));
        //}

        LineCap GetStartCap()
        {
            LineCap startCap;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenStartCap(nativePen, out startCap));

            return startCap;
        }

        LineCap GetEndCap()
        {
            LineCap endCap;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenEndCap(nativePen, out endCap));

            return endCap;
        }

        DashCap GetDashCap()
        {
            DashCap dashCap;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenDashCap197819(nativePen,
                                out dashCap));

            return dashCap;
        }

        void SetLineJoin(LineJoin lineJoin)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenLineJoin(nativePen, lineJoin));
        }

        LineJoin GetLineJoin()
        {
            LineJoin lineJoin;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenLineJoin(nativePen, out lineJoin));

            return lineJoin;
        }

        void SetCustomStartCap(CustomLineCap customCap)
        {
            GpCustomLineCap nativeCap = new GpCustomLineCap();
            if (customCap != null)
                nativeCap = customCap.nativeCap;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenCustomStartCap(nativePen,
                                                                  nativeCap));
        }

        CustomLineCap GetCustomStartCap()
        {
            CustomLineCap customCap = new CustomLineCap();
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenCustomStartCap(nativePen,
                                                        out customCap.nativeCap));
            return customCap;
        }

        void SetCustomEndCap(CustomLineCap customCap)
        {
            GpCustomLineCap nativeCap = new GpCustomLineCap();
            if (customCap != null)
                nativeCap = customCap.nativeCap;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenCustomEndCap(nativePen,
                                                                nativeCap));
        }

        CustomLineCap GetCustomEndCap()
        {
            CustomLineCap customCap = new CustomLineCap();
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenCustomEndCap(nativePen,
                                                        out customCap.nativeCap));
            return customCap;
        }

        void SetMiterLimit(float miterLimit)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenMiterLimit(nativePen,
                                                        miterLimit));
        }

        float GetMiterLimit()
        {
            float miterLimit;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenMiterLimit(nativePen, out miterLimit));

            return miterLimit;
        }

        void SetAlignment(PenAlignment penAlignment)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenMode(nativePen, penAlignment));
        }

        PenAlignment GetAlignment()
        {
            PenAlignment penAlignment;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenMode(nativePen, out penAlignment));

            return penAlignment;
        }

        //PenType GetPenType()
        //{
        //    PenType type;
        //    SetStatus(NativeMethods.GdipGetPenFillType(nativePen, out type));

        //    return type;
        //}

        void SetColor(Color color)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenColor(nativePen,
                                                         color.ToArgb()));
        }

        public void SetBrush(BrushPlus brush)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenBrushFill(nativePen,
                                           brush.nativeBrush));
        }

        Color GetColor()
        {

            int argb;
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenColor(nativePen,  out argb));
            return Color.FromArgb(argb);
        }

        public DashStyle GetDashStyle()
        {
            DashStyle dashStyle;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenDashStyle(nativePen, out dashStyle));

            return dashStyle;
        }

        public void SetDashStyle(DashStyle dashStyle)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenDashStyle(nativePen,
                                                             dashStyle));
        }

        float GetDashOffset()
        {
            float dashOffset;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenDashOffset(nativePen, out dashOffset));

            return dashOffset;
        }

        void SetDashOffset(float dashOffset)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenDashOffset(nativePen,
                                                              dashOffset));
        }

        public void SetDashPattern(float[] dashArray)
        {
            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipSetPenDashArray(nativePen,
                                                             dashArray,
                                                             dashArray.Length));
        }

        int GetDashPatternCount()
        {
            int count = 0;

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenDashCount(nativePen, out count));

            return count;
        }

        public void GetDashPattern(float[] dashArray)
        {
            if (dashArray == null || dashArray.Length == 0)
                SetStatus(GpStatus.InvalidParameter);

            SetStatus( Silvermoon.GdiPlus.Native.GdiPlus.GdipGetPenDashArray(nativePen,
                                                             dashArray,
                                                             dashArray.Length));
        }


        public GpStatus GetLastStatus()
        {
            GpStatus lastStatus = lastResult;
            lastResult = GpStatus.Ok;

            return lastStatus;
        }




        protected PenPlus(GpPen nativePen, GpStatus status)
        {
            lastResult = status;
            SetNativePen(nativePen);
        }

        void SetNativePen(GpPen nativePen)
        {
            this.nativePen = nativePen;
        }


        internal GpPen nativePen;
        protected GpStatus lastResult;

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
            }
            // free native resources if there are any.
            if ((IntPtr)nativePen!= IntPtr.Zero)
            {
                 Silvermoon.GdiPlus.Native.GdiPlus.GdipDeletePen(nativePen);
                nativePen = new GpPen();
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
